@@ -1,6 +1,7 @@
 import sys
 import typing
 import asyncio
+import logging
 import traceback
 from contextlib import suppress
 
@@ -71,8 +72,14 @@ class Client(APIClient):
         self.__ready_future = asyncio.Future()
         self.monoshard: bool = monoshard
         self.shard_count: typing.Optional[int] = shard_count
+        self.logger = logging.getLogger("dico.client")
         self.__shards = {} if self.monoshard else None
         self.__shard_id = shard_id
+        self.__shard_ids = []
+        self.__shard_ids_ready = []
+        self.__unavailable_guilds = set()
+        self.__compress = False
+        self.__reconnect_on_unknown_disconnect = False
 
         # Internal events dispatch
         self.events.add("READY", self.__ready)
@@ -80,8 +87,19 @@ class Client(APIClient):
 
     def __ready(self, ready):
         self.application_id = Snowflake(ready.application["id"])
-        if not self.__ready_future.done():
-            self.__ready_future.set_result(True)
+        if not self.__shards:
+            if not self.__ready_future.done():
+                self.__ready_future.set_result(True)
+            self.__unavailable_guilds = set(x["id"] for x in ready.guilds)
+        else:
+            if ready.shard_id not in self.__shard_ids_ready:
+                self.__shard_ids_ready.append(ready.shard_id)
+                self.__unavailable_guilds.update([x["id"] for x in ready.guilds])
+            if self.__shard_ids == self.__shard_ids_ready:
+                self.__shard_ids_ready = []
+                if not self.__ready_future.done():
+                    self.__ready_future.set_result(True)
+                self.dispatch("shards_ready")
 
     def __voice_state_update(self, voice_state):
         if self.has_cache:
@@ -224,13 +242,15 @@ class Client(APIClient):
             shard_count = self.shard_count or gateway.shards
             if self.shard_count is None:
                 self.shard_count = gateway.shards
-            for x in range(shard_count):
+            self.__shard_ids = [*range(shard_count)]
+            for x in self.__shard_ids:
                 ws = await self.__ws_class.connect_without_request(
                     gateway, self.http, self.intents, self.events, reconnect_on_unknown_disconnect, compress, shard=[x, shard_count]
                 )
                 self.__shards[x] = ws
                 await ws.receive_once()
                 self.loop.create_task(ws.run())
+                await asyncio.sleep(5)
         else:
             maybe_shard = {"shard": [self.__shard_id, self.shard_count]} if self.__shard_id else {}
             self.ws = await self.__ws_class.connect(self.http, self.intents, self.events, reconnect_on_unknown_disconnect, compress, **maybe_shard)
@@ -287,6 +307,28 @@ class Client(APIClient):
             raise AttributeError(f"shard for guild {int(guild)} not found.")
         return ws.update_voice_state(str(int(guild)), str(int(channel)) if channel else None, self_mute, self_deaf)
 
+    async def increase_shards(self, number: int = 1, *, auto: bool = False):
+        """
+        Increases numbers of shards.
+
+        .. warning::
+            During this action, client will be offline.
+
+        :param int number: Number of shards to increase. Default 1.
+        :param bool auto: Whether to automatically use recommended number. Default False.
+        """
+        if self.ws:
+            raise TypeError("this client is not monosharded.")
+        elif self.__shards:
+            for x in self.__shards.values():
+                await x.close()
+        if auto:
+            resp = await self.request_gateway()
+            self.shard_count = resp.shards
+        else:
+            self.shard_count += number
+        self.loop.create_task(self.start(reconnect_on_unknown_disconnect=self.__reconnect_on_unknown_disconnect, compress=self.__compress))
+
     @property
     def has_cache(self) -> bool:
         """Whether the caching is enabled."""
@@ -310,9 +352,12 @@ class Client(APIClient):
 
     @property
     def guild_count(self) -> typing.Optional[int]:
-        """Total count of guilds this bot is in."""
+        """Total count of guilds this bot is in. This may be incorrect since this relies on initial guild count or cached guild count."""
+        initial_guild_count = len(self.__unavailable_guilds)
         if self.has_cache:
-            return self.cache.get_size("guild")
+            return sorted([initial_guild_count, self.cache.get_size("guild")])[-1]
+        self.logger.warning("Caching is disabled, showing initial guild count.")
+        return initial_guild_count
 
     @property
     def ping(self) -> float:
@@ -362,6 +407,8 @@ class Client(APIClient):
         :param bool reconnect_on_unknown_disconnect: Whether to reconnect on unknown websocket error.
         :param bool compress: Whether to enable zlib compress.
         """
+        self.__reconnect_on_unknown_disconnect = self.__reconnect_on_unknown_disconnect
+        self.__compress = compress
         try:
             self.loop.create_task(self.start(reconnect_on_unknown_disconnect, compress))
             self.loop.run_forever()
@@ -372,3 +419,7 @@ class Client(APIClient):
             traceback.print_exc()
         finally:
             self.loop.run_until_complete(self.close())
+
+    def kill(self):
+        """Kills event loop and stops bot."""
+        self.loop.stop()
