@@ -28,3 +28,6 @@ class Emoji:
 
     def __int__(self) -> int:
         return int(self.id)
+
+    def __repr__(self) -> str:
+        return f"<{self.__class__.__name__} id={self.id} name={self.name}>"
