@@ -1,3 +1,4 @@
+import sys
 import typing
 import datetime
 from .application import Application
@@ -26,7 +27,7 @@ class Ready(EventBase):
         self.user: User = User.create(client, resp["user"])
         self.guilds: typing.List[dict] = resp["guilds"]
         self.session_id: str = resp["session_id"]
-        self.shard: typing.Optional[list] = resp.get("shard")
+        self.shard: typing.Optional[typing.List[int]] = resp.get("shard")
         self.application: dict = resp["application"]
 
     @property
@@ -37,6 +38,11 @@ class Ready(EventBase):
     def guild_count(self) -> int:
         return len(self.guilds)
 
+    @property
+    def shard_id(self):
+        if self.shard:
+            return self.shard[0]
+
 
 """
 class ApplicationCommandCreate(ApplicationCommand):
@@ -153,6 +159,9 @@ class GuildCreate(Guild):
     @classmethod
     def create(cls, client: "Client", resp: dict, **kwargs):
         kwargs.setdefault("ensure_cache_type", "guild")
+        if "name" not in resp:
+            print(f"Invalid payload warning! Full payload: {resp}", file=sys.stderr)
+            return None
         return super().create(client, resp, **kwargs)
 
 
