@@ -33,6 +33,9 @@ class StageInstance(DiscordObjectBase):
         if self.client.has_cache:
             return self.client.get(self.channel_id, "channel")
 
+    def __repr__(self) -> str:
+        return f"<{self.__class__.__name__} id={self.id}>"
+
 
 class PrivacyLevel(TypeBase):
     PUBLIC = 1
