@@ -15,6 +15,7 @@ class WebSocketClient:
     ZLIB_SUFFIX = b'\x00\x00\xff\xff'
     RECONNECT_CODES = [1000, 1001, 1006, 4000, 4001, 4002, 4003, 4005, 4007, 4009]
     INPUT_WARN = [4001, 4002, 4003, 4005]
+    WS_KWARGS = {"autoclose": False, "autoping": False, "timeout": 60}
 
     def __init__(self,
                  http: AsyncHTTPRequest,
@@ -22,29 +23,41 @@ class WebSocketClient:
                  base_url: str,
                  intents: typing.Union[gateway.Intents, int],
                  event_handler: EventHandler,
-                 try_reconnect: bool):
-        self.ws = ws
-        self.http = http
-        self.base_url = base_url
-        self.event_handler = event_handler
-        self.logger = logging.getLogger("dico.ws")
-        self._closed = False
-        self._reconnecting = False
-        self._fresh_reconnecting = False
-        self.heartbeat_interval = 0
-        self.seq = None
-        self.last_heartbeat_ack = 0
-        self.last_heartbeat_send = 0
-        self._heartbeat_task = None
-        self.session_id = None
-        self.intents = intents
-        self.ping = 0.0
-        self._ping_start = 0.0
-        self.try_reconnect = try_reconnect
-        self.ratelimit = WSRatelimit()
-        self.buffer = bytearray()
+                 try_reconnect: bool,
+                 compress: typing.Optional[bool] = None,
+                 large_threshold: typing.Optional[int] = None,
+                 shard: typing.Optional[typing.List[int]] = None,
+                 presence: typing.Optional[dict] = None):
+        self.ws: aiohttp.ClientWebSocketResponse = ws
+        self.http: AsyncHTTPRequest = http
+        self.base_url: str = base_url
+        self.event_handler: EventHandler = event_handler
+        logger_name = "dico.ws"
+        if shard:
+            logger_name += f".shard{shard[0]}"
+        self.logger: logging.Logger = logging.getLogger(logger_name)
+        self._closed: bool = False
+        self._reconnecting: bool = False
+        self._fresh_reconnecting: bool = False
+        self.heartbeat_interval: int = 0
+        self.seq: typing.Optional[int] = None
+        self.last_heartbeat_ack: float = 0
+        self.last_heartbeat_send: float = 0
+        self._heartbeat_task: typing.Optional[asyncio.Task] = None
+        self.session_id: typing.Optional[str] = None
+        self.intents: gateway.Intents = intents
+        self.ping: float = 0.0
+        self._ping_start: float = 0.0
+        self.try_reconnect: bool = try_reconnect
+        self.ratelimit: WSRatelimit = WSRatelimit()
+        self.buffer: bytearray = bytearray()
         self.inflator = zlib.decompressobj()
-        self.intended_shutdown = False
+        self.intended_shutdown: bool = False
+
+        self.compress: typing.Optional[bool] = compress
+        self.large_threshold: typing.Optional[int] = large_threshold
+        self.shard: typing.Optional[typing.List[int]] = shard
+        self.presence: typing.Optional[dict] = presence
 
     def __del__(self):
         if not self._closed:
@@ -52,8 +65,7 @@ class WebSocketClient:
             self.http.loop.run_until_complete(self.close())
 
     async def close(self, code: int = 1000):
-        if not self._heartbeat_task.cancelled():
-            self._heartbeat_task.cancel()
+        await self.cancel_heartbeat()
         self.intended_shutdown = True
         if self._closed:
             return
@@ -84,6 +96,7 @@ class WebSocketClient:
                         self.logger.warning("Trying to reconnect...")
                         await self.reconnect(fresh=True)
                     else:
+                        await self.close(4000)
                         if self.intended_shutdown:
                             self.logger.info("This client will now be terminated.")
                         else:
@@ -109,7 +122,7 @@ class WebSocketClient:
                     break
 
             if self._reconnecting or self._fresh_reconnecting:
-                self.ws = await self.http.session.ws_connect(self.base_url)
+                self.ws = await self.http.session.ws_connect(self.base_url, **self.WS_KWARGS)
                 self._closed = False
                 self.intended_shutdown = False
             else:
@@ -121,6 +134,11 @@ class WebSocketClient:
                     continue
                 return
 
+    async def receive_once(self):
+        msg = await self.ws.receive()
+        resp = await self.receive(msg)
+        await self.process(resp)
+
     async def request(self, *args, **kwargs):
         async with self.ratelimit.maybe_limited():  # Will raise exception if rate limited, else wait until previous request is done.
             return await self.ws.send_json(*args, **kwargs)
@@ -131,7 +149,7 @@ class WebSocketClient:
             self.seq = res.s
         return res
 
-    async def receive(self, resp: aiohttp.WSMessage):
+    async def receive(self, resp: aiohttp.WSMessage) -> gateway.GatewayResponse:
         # resp = await self.ws.receive()
         self.logger.debug(f"Raw receive {resp.type}: {resp.data}")
         if resp.type == aiohttp.WSMsgType.TEXT:
@@ -149,7 +167,7 @@ class WebSocketClient:
         elif resp.type in (aiohttp.WSMsgType.CLOSE, aiohttp.WSMsgType.CLOSED, aiohttp.WSMsgType.CLOSING):
             raise WSClosing(resp.data or self.ws.close_code)
 
-    async def process(self, resp):
+    async def process(self, resp) -> typing.Optional[int]:
         if resp.op == gateway.Opcodes.DISPATCH:
             if resp.t == "READY":
                 self.session_id = resp.d.get("session_id", self.session_id)
@@ -190,8 +208,7 @@ class WebSocketClient:
         if self._reconnecting or self._fresh_reconnecting:
             self.logger.warning("Reconnection is already running, but another reconnection is requested. This request is ignored.")
             return
-        if not self._heartbeat_task.cancelled():
-            self._heartbeat_task.cancel()
+        await self.cancel_heartbeat()
         self._reconnecting = not fresh
         self._fresh_reconnecting = fresh
         self.logger.info("Reconnecting to Websocket...")
@@ -211,20 +228,34 @@ class WebSocketClient:
         self._reconnecting = False
 
     async def run_heartbeat(self):
-        while not self._closed:
-            if self._reconnecting or self._fresh_reconnecting:
-                break  # Just making sure
-            if not self.last_heartbeat_send <= self.last_heartbeat_ack <= time.time():
+        try:
+            while not self._closed:
                 if self._reconnecting or self._fresh_reconnecting:
+                    break  # Just making sure
+                if not self.last_heartbeat_send <= self.last_heartbeat_ack <= time.time():
+                    if self._reconnecting or self._fresh_reconnecting:
+                        break
+                    self.logger.warning("Heartbeat timeout, reconnecting...")
+                    self.http.loop.create_task(self.reconnect())
                     break
-                self.logger.warning("Heartbeat timeout, reconnecting...")
-                self.http.loop.create_task(self.reconnect())
-                break
-            data = {"op": gateway.Opcodes.HEARTBEAT, "d": self.seq}
-            self._ping_start = time.time()
-            await self.ws.send_json(data)  # Don't track rate limit for this.
-            self.last_heartbeat_send = time.time()
-            await asyncio.sleep(self.heartbeat_interval/1000)
+                data = {"op": gateway.Opcodes.HEARTBEAT, "d": self.seq}
+                self._ping_start = time.time()
+                await self.ws.send_json(data)  # Don't track rate limit for this.
+                self.last_heartbeat_send = time.time()
+                await asyncio.sleep(self.heartbeat_interval / 1000)
+        except asyncio.CancelledError:
+            return
+
+    async def cancel_heartbeat(self):
+        if self._heartbeat_task.cancelled():
+            return
+        self.last_heartbeat_ack = 0
+        self.last_heartbeat_send = 0
+        self._heartbeat_task.cancel()
+        try:
+            await self._heartbeat_task
+        except asyncio.CancelledError:
+            pass
 
     async def identify(self):
         data = {
@@ -239,6 +270,14 @@ class WebSocketClient:
                 }
             }
         }
+        if self.compress is not None:
+            data["d"]["compress"] = self.compress
+        if self.large_threshold is not None:
+            data["d"]["large_threshold"] = self.large_threshold
+        if self.shard is not None:
+            data["d"]["shard"] = self.shard
+        if self.presence is not None:
+            data["d"]["presence"] = self.presence
         await self.request(data)
 
     async def request_guild_members(self, guild_id: str, *, query: str = None, limit: int = None, presences: bool = None, user_ids: typing.List[str] = None, nonce: bool = None):
@@ -283,23 +322,47 @@ class WebSocketClient:
         await self.request(data)
 
     @property
-    def closed(self):
+    def closed(self) -> bool:
         return self._closed
 
     @classmethod
-    async def connect(cls, http, intents, event_handler, try_reconnect, compress):
+    async def connect(cls,
+                      http,
+                      intents,
+                      event_handler,
+                      try_reconnect,
+                      compress,
+                      large_threshold: typing.Optional[int] = None,
+                      shard: typing.Optional[typing.List[int]] = None,
+                      presence: typing.Optional[dict] = None):
         resp = await http.request("/gateway/bot", "GET")
         gw = gateway.GetGateway(resp)
         extra = "compress=zlib-stream" if compress else ""
         base_url = gw.url+f"?v=9&encoding=json" + extra
-        ws = await http.session.ws_connect(base_url)
-        return cls(http, ws, base_url, intents, event_handler, try_reconnect)
+        ws = await http.session.ws_connect(base_url, **cls.WS_KWARGS)
+        return cls(http, ws, base_url, intents, event_handler, try_reconnect, None, large_threshold, shard, presence)
+
+    @classmethod
+    async def connect_without_request(cls,
+                                      gw_response: gateway.GetGateway,
+                                      http,
+                                      intents,
+                                      event_handler,
+                                      try_reconnect,
+                                      compress,
+                                      large_threshold: typing.Optional[int] = None,
+                                      shard: typing.Optional[typing.List[int]] = None,
+                                      presence: typing.Optional[dict] = None):
+        extra = "compress=zlib-stream" if compress else ""
+        base_url = gw_response.url+f"?v=9&encoding=json" + extra
+        ws = await http.session.ws_connect(base_url, **cls.WS_KWARGS)
+        return cls(http, ws, base_url, intents, event_handler, try_reconnect, None, large_threshold, shard, presence)
 
 
 class WSClosing(Exception):
     """Internal exception class for controlling WS close event. You should not see this exception in normal situation."""
-    def __init__(self, code):
-        self.code = code
+    def __init__(self, code: int):
+        self.code: int = code
 
 
 class Ignore(Exception):
